
#include <AppKit.h>
#include <InterfaceKit.h>
#include <StorageKit.h>
#include <TranslationKit.h>

const char* kAppSignature = "application/x-vnd.Public-ImageDrop3";

//---------------------------------------------------------------------------------
// ImageDropView

class ImageDropView : public BView
{
private:
	char     fImageFile[1024];
	BBitmap* fImg[2];
	int      fIndex;

public :
	ImageDropView(BRect frame, const char* name)
		:
		BView(frame, name, B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS)
	{
		fImg[0] = NULL;
		fImg[1] = NULL;
		fIndex = 0;
	}
	
	~ImageDropView()
	{
		if (fImg[0])
			delete fImg[0];
		if (fImg[1])
			delete fImg[1];
	}
	
	void MessageReceived(BMessage *msg)
	{
		switch (msg->what) {
			case 'draw':
			{
				const char* filename;
				if (msg->FindString("imagefile", &filename) == B_OK) {
					// set image filename and force re-paint
					strcpy(fImageFile, filename);
					Invalidate();
				}
			}
			break;
			
			default:
				BView::MessageReceived(msg);
		}
	}
	
	void FrameResized (float width, float height)
	{
		Invalidate();
	}
	
	void Draw(BRect frame)
	{
		int i = fIndex;
		
		fImg[i] = BTranslationUtils::GetBitmapFile(fImageFile);
		if (fImg[i]) {
			// draw the "fresh" bitmap
			BRect rect = Bounds();
			rect.InsetBy(5,5);
			DrawBitmap(fImg[i], rect);
		}
		
		// toggle index between 0 and 1
		if (++fIndex == 2)	
			fIndex = 0;
		
		// delete the "stale" bitmap
		if (fImg[fIndex])	
			delete fImg[fIndex];
	}
};


//---------------------------------------------------------------------------------
// ImageDropWindow

class ImageDropWindow : public BWindow
{
private:
	ImageDropView* fView;

public:
	ImageDropWindow(BRect frame) : BWindow(frame, NULL, B_DOCUMENT_WINDOW, 0)
	{
		SetTitle("ImageDrop3");
		fView = new ImageDropView(Bounds(), "fView");
		AddChild(fView);
		Show();
	}
	
	~ImageDropWindow()
	{
		BMessenger(kAppSignature).SendMessage(new BMessage('quit'));
	}
	
	void MessageReceived(BMessage* msg)
	{
		if (msg->WasDropped()) {
			// get dropped item
			entry_ref ref;
			if (msg->FindRef("refs", 0, &ref) == B_OK) {
				BEntry entry;
				if (entry.SetTo(&ref) == B_OK) {
					// get full path of filename
					BPath path(&entry);
					entry.GetPath(&path);
					
					// tell the view to draw it
					BMessage* msg = new BMessage('draw');
					msg->AddString("imagefile", path.Path());
					BMessenger(fView).SendMessage(msg);
					return;
				}
			}
		}
		else
			BWindow::MessageReceived(msg);
	}
};


//---------------------------------------------------------------------------------
// ImageDrop

class ImageDrop : public BApplication
{
public:
	ImageDrop() : BApplication(kAppSignature)
	{
		new ImageDropWindow(BRect(90,120,1020,860));
	}
	
	void MessageReceived (BMessage* msg)
	{
		switch (msg->what) {
			case 'quit':
				BApplication::Quit();
				break;
			default:
				BApplication::MessageReceived(msg);
		}
	}
};


int main()
{
	ImageDrop app;
	
	app.Run();
	return 0;
}

